/****************************************************************************
**
** Copyright (C) 2016 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtSensors module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:GPL-EXCEPT$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QStringList>
#include <QTimer>

#include "qtestrecognizer.h"
#include "qtestsensorgestureplugin_p.h"


QTestRecognizer::QTestRecognizer(QObject *parent)
    : QSensorGestureRecognizer(parent),
      active(0)
{
    timeout();
}

QTestRecognizer::~QTestRecognizer()
{
}

void QTestRecognizer::timeout()
{
    Q_EMIT detected("tested");
    Q_EMIT tested();
    QTimer::singleShot(100, this, SLOT(timeout()));
}


bool QTestRecognizer::start()
{
    Q_EMIT detected("tested");
    Q_EMIT tested();
    active = true;
    return true;
}

bool QTestRecognizer::stop()
{
    active = false;
    return true;
}

bool QTestRecognizer::isActive()
{
    return active;
}

void  QTestRecognizer::create()
{
    active = false;
}

QString QTestRecognizer::id() const
{
    return QString("QtSensors.test");
}


int QTestRecognizer::thresholdTime() const
{
    return timerTimeout;
}

void QTestRecognizer::setThresholdTime(int msec)
{
    timer->setInterval(msec);
}
