/****************************************************************************
**
** Copyright (C) 2019 BogDan Vatra <bogdan@kde.org>
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtSensors module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef SENSORMANAGER_H
#define SENSORMANAGER_H

#include <QThread>
#include <QSemaphore>
#include <QMutex>
#include <QWaitCondition>

#include <private/qjni_p.h>
#include <private/qjnihelpers_p.h>

#include <android/sensor.h>

class SensorManager : public QThread
{
public:
    ~SensorManager() override;
    static QSharedPointer<SensorManager> &instance();
    ALooper *looper() const;
    ASensorManager *manager() const;

    QJNIObjectPrivate javaSensor(const ASensor *sensor) const;
    QString description(const ASensor *sensor) const;
    double getMaximumRange(const ASensor *sensor) const;

private:
    SensorManager();
    // QThread interface
    void run() override;

private:
    QAtomicInt m_quit{0};
    ALooper *m_looper = nullptr;
    QSemaphore m_waitForStart;
    QJNIObjectPrivate m_sensorManager;
};

#endif // SENSORMANAGER_H
